from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *
from collections import namedtuple

AlembicConfiguration = namedtuple('AlembicConfiguration', ['alembic_directory'])
